@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/aide.h,v 1.15 2006/10/06 13:12:33 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@
 #define _AIDE_H_INCLUDED
 
 #include "config.h"
-#include "report.h"
+#include "types.h"
 #include "db_config.h"
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,6 +31,7 @@
 #if HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
+#include "report.h"
 
 #ifndef TEMP_FAILURE_RETRY
 /* Evaluate EXPRESSION, and repeat as long as it returns -1 with errno'
@@ -44,22 +45,15 @@
         __result; }))
 #endif
 
-#ifdef HAVE_SNPRINTF
-#if !defined(HAVE_C99_SNPRINTF) || !defined(HAVE_C99_VSNPRINTF)
-#define PREFER_PORTABLE_SNPRINTF
-#endif
+#if !defined HAVE_VSNPRINTF || !defined HAVE_C99_VSNPRINTF
+#define vsnprintf rsync_vsnprintf
+int vsnprintf(char *str, size_t count, const char *fmt, va_list args);
 #endif
 
-#ifdef HAVE_VSNPRINTF
-#ifndef HAVE_SNPRINTF
-#define HAVE_SNPRINTF
-#define PREFER_PORTABLE_SNPRINTF
+#if !defined HAVE_SNPRINTF || !defined HAVE_C99_VSNPRINTF
+#define snprintf rsync_snprintf
+int snprintf(char *str,size_t count,const char *fmt,...);
 #endif
-#endif
-
-#define SNPRINTF_LONGLONG_SUPPORT
-
-#include "snprintf.h"
 
 #ifndef O_NOATIME
 #if defined(__linux__) && (defined(__i386__) || defined(__PPC__))
@@ -73,7 +67,7 @@
 # define HAVE_STRTOIMAX
 #endif
 
-#if AIDE_OFF_TYPE == off64_t
+#if SIZEOF_OFF64_T == SIZEOF_LONG_LONG
 # ifdef HAVE_STRTOLL
 #  define AIDE_STRTOLL_FUNC strtoll
 # else
