@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/compare_db.h,v 1.2 2006/10/27 18:54:29 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@ void compare_db(list*,db_config*);
  * 1: total number of files 2: # added files, 
  * 3: # removed, 4: # changed
  */
-long report_tree(seltree* tree,int stage, long* stat);
+long report_tree(seltree* tree,int stage, long* status);
 
 void init_rxlst(list* rxlst);
 
@@ -48,6 +48,6 @@ void init_rxlst(list* rxlst);
  * compare_dbline()
  * Return RETOK if same RETFAIL if not
  */
-int compare_dbline(db_line* old,db_line* new,DB_ATTR_TYPE ignorelist);
+DB_ATTR_TYPE compare_dbline(db_line* old,db_line* new,DB_ATTR_TYPE ignorelist);
 
 #endif
