@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/db_config.h,v 1.12 2006/10/27 21:16:57 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,12 +21,12 @@
 #ifndef _DB_CONFIG_H_INCLUDED
 #define _DB_CONFIG_H_INCLUDED
 #include "aide.h"
+#include "types.h"
 #include <unistd.h>
 #include <stdio.h>
 
 #define E2O(n) (1<<n)
 
-#include "types.h"
 #include "list.h"
 #include "seltree.h"
 
@@ -80,6 +80,9 @@ typedef struct xattrs_type
 
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
+#ifndef ENOATTR
+# define ENOATTR ENODATA 
+#endif
 #endif
 
 #ifdef WITH_MHASH
@@ -335,7 +338,9 @@ typedef struct db_config {
   int no_acl_on_symlinks;
 #endif
   int warn_dead_symlinks;
-  
+
+  int summarize_changes;
+
   struct seltree* tree;
 
 } db_config;
