@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/db_list.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,13 +20,15 @@
 
 #ifndef _DB_LIST_H_INCLUDED
 #define _DB_LIST_H_INCLUDED
+
+#include <stdio.h>	/* for NULL */
+
 typedef struct db_list{
-  dbline* data
-  db_list* next
-  db_list* prev
-  db_list* head
-  db_list* tail
-} db_list
+  struct db_list* next;
+  struct db_list* prev;
+  struct db_list* head;
+  struct db_list* tail;
+} db_list;
 
 /* Only the head knows for sure where the tail is */
 static db_list* db_list_head=NULL;
