@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/report.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include "db_config.h"
+#include "url.h"
 
 /* Exitcodes */
 #define ERROR_WRITING_ERROR 14
@@ -31,11 +32,17 @@
 #define UNIMPLEMENTED_FUNCTION_ERROR 16
 #define INVALID_CONFIGURELINE_ERROR 17
 #define IO_ERROR 18
+#define VERSION_MISMATCH_ERROR 19
 
 /* Errorcodes */
 #define HASH_ALGO_ERROR 30
 
-void error(int errorlevel, char* error_msg,...);
+void error(int errorlevel, char* error_msg,...)
+#ifdef __GNUC__
+        __attribute__ ((format (printf, 2, 3)));
+#else
+ ;
+#endif
 
 int error_init(url_t*,int);
 
