@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/aide.c,v 1.26 2006/11/30 19:41:52 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -65,7 +65,7 @@ void usage(int exitvalue)
 	    "  -u, --update\t\tCheck and update the database non-interactively\n"
 	    "      --compare\t\tCompare two databases\n\n"
 	    "Miscellaneous:\n"
-	    "      --config-check\tTest the configuration file\n"
+	    "  -D, --config-check\tTest the configuration file\n"
 	    "  -v, --version\t\tShow version of AIDE and compilation options\n"
 	    "  -h, --help\t\tShow this help message\n\n"
 	    "Options:\n"
@@ -325,6 +325,8 @@ void setdefaults_before_config()
 
   conf->warn_dead_symlinks=0;
 
+  conf->summarize_changes=0;
+
   conf->selrxlst=NULL;
   conf->equrxlst=NULL;
   conf->negrxlst=NULL;
@@ -544,7 +546,7 @@ int main(int argc,char**argv)
   if (!conf->config_check) {
     int exitcode=0;
     long totalchanges=0;
-    long stat[5]={1,0,0,0,0};
+    long status[5]={1,0,0,0,0};
     if(conf->action&DO_INIT){
       if(db_init(DB_WRITE)==RETFAIL) {
 	exit(IO_ERROR);
@@ -574,18 +576,18 @@ int main(int argc,char**argv)
     db_close(conf);
 
     /* First terse report */
-    totalchanges=report_tree(conf->tree,0,stat);
+    totalchanges=report_tree(conf->tree,0,status);
     if(totalchanges>0){
-      exitcode=(stat[2]!=0)*1+(stat[3]!=0)*2+(stat[4]!=0)*4;
-      stat[0]=1;
-      report_tree(conf->tree,1,stat);
-      stat[0]=1;
-      report_tree(conf->tree,2,stat);
-      stat[0]=1;
-      report_tree(conf->tree,3,stat);
+      exitcode=(status[2]!=0)*1+(status[3]!=0)*2+(status[4]!=0)*4;
+      status[0]=1;
+      report_tree(conf->tree,1,status);
+      status[0]=1;
+      report_tree(conf->tree,2,status);
+      status[0]=1;
+      report_tree(conf->tree,3,status);
       /* Then detailed list of changes */
-      stat[0]=1;
-      report_tree(conf->tree,4,stat);
+      status[0]=1;
+      report_tree(conf->tree,4,status);
     } else {
       if (conf->verbose_level >= 5) {
         printf("\nAIDE, version " AIDEVERSION "\n\n");
@@ -609,9 +611,9 @@ int main(int argc,char**argv)
       error(0,"Config checked. Use the following to patch your config file.\n");
       error(0,"0a1\n");
       if(newlinelastinconfig==1){
-	error(0,"> @@begin_config %s\n%ia%i\n> @@end_config\n",digstr,conf_lineno-1,conf_lineno+1);
+	error(0,"> @@begin_config %s\n%lia%li\n> @@end_config\n",digstr,conf_lineno-1,conf_lineno+1);
       }else {
-	error(0,"> @@begin_config %s\n%ia%i\n> @@end_config\n",digstr,conf_lineno,conf_lineno+2);
+	error(0,"> @@begin_config %s\n%lia%li\n> @@end_config\n",digstr,conf_lineno,conf_lineno+2);
       }
       free(dig);
       free(digstr);
