@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/be.c,v 1.10 2006/12/14 16:09:24 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -161,7 +161,7 @@ FILE* be_init(int inout,url_t* u,int iszipped)
 #endif
 #ifdef WITH_ZLIB
     if(iszipped && !inout){
-      fh=gzdopen(fd,"wb9+");
+      fh=gzdopen(fd,"wb9");
       if(fh==NULL){
 	error(0,_("Couldn't open file %s for %s"),u->value,
 	      inout?"reading\n":"writing\n");
@@ -224,14 +224,14 @@ FILE* be_init(int inout,url_t* u,int iszipped)
     if(iszipped && !inout){
       fh=gzdopen(a,"w");
       if(fh==NULL){
-	error(0,"Couldn't reopen file descriptor %i\n",a);
+	error(0,"Couldn't reopen file descriptor %li\n",a);
       }
     }
     else{
 #endif
       fh=fdopen(a,inout?"r":"w");
       if(fh==NULL){
-	error(0,"Couldn't reopen file descriptor %i\n",a);
+	error(0,"Couldn't reopen file descriptor %li\n",a);
       }
 #ifdef WITH_ZLIB
     }
