@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/commandconf.c,v 1.19 2006/12/14 16:09:24 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -84,7 +84,7 @@ int commandconf(const char mode,const char* line)
     else {
       tmp=(char*)malloc(sizeof(char)
 			*(strlen(after)+strlen(line)+2));
-      strcat(tmp,after);
+      strcpy(tmp,after);
       strcat(tmp,"\n");
       strcat(tmp,line);
       free(after);
@@ -216,7 +216,7 @@ int conf_input_wrapper(char* buf, int max_size, FILE* in)
 	    MHASH_FAILED){
 	  error(0, "mhash_hmac_init() failed for %i for config check. Aborting\n",
 		conf->confhmactype);
-	  abort();
+	  exit(EXIT_FAILURE);
 	}
       } else {
 	conf->do_configmd=0;
@@ -314,9 +314,9 @@ int db_input_wrapper(char* buf, int max_size, int db)
 	buf[0]='\0';
       } else {
 	/* gzread returns 0 even if uncompressed bytes were read*/
-	error(240,"nread=%d,strlen(buf)=%d,errno=%s,gzerr=%s\n",
-              retval,strnlen((char*)buf, max_size),strerror(errno),
-	      gzerror(*db_gzp,&err));
+	error(240,"nread=%d,strlen(buf)=%lu,errno=%s,gzerr=%s\n",
+              retval,(unsigned long)strnlen((char*)buf, max_size),
+              strerror(errno),gzerror(*db_gzp,&err));
 	if(retval==0){
 	  retval=strnlen((char*)buf, max_size);
 	}
@@ -341,7 +341,7 @@ int db_input_wrapper(char* buf, int max_size, int db)
   if(c==-1) {
     int xx;
 	  error(0,"Error reading gzipped file: %s\n",gzerror(*db_gzp,&xx));
-    abort();
+    exit(EXIT_FAILURE);
   }
       }else {
 	/* False alarm */
@@ -377,7 +377,7 @@ int db_input_wrapper(char* buf, int max_size, int db)
 			     mhash_get_hash_pblock(conf->dbhmactype)))==
 	    MHASH_FAILED){
 	  error(0, "mhash_hmac_init() failed for db check. Aborting\n");
-	  abort();
+	  exit(EXIT_FAILURE);
 	}
       } else {
 	*domd=0;
@@ -722,7 +722,7 @@ list* append_rxlist(char* rx,DB_ATTR_TYPE attr,list* rxlst)
   r->attr=attr;
   r->conf_lineno = conf_lineno;
   if (attr&DB_CHECKINODE && attr&DB_CTIME)
-    error(20,"Rule at line %i has c and I flags enabled at the same time. If same inode is found, flag c is ignored\n",conf_lineno);
+    error(20,"Rule at line %li has c and I flags enabled at the same time. If same inode is found, flag c is ignored\n",conf_lineno);
   update_db_out_order(r->attr);
   rxlst=list_append(rxlst,(void*)r);
   
