@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999-2006 Rami Lehti, Pablo Virolainen, Richard van den Berg
- * $Id: compare_db.c,v 1.19 2006/10/27 20:44:38 rvdb Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -53,9 +53,10 @@
 const int old_col  = 12;   
 const int new_col  = 40;   
 
-const int part_len = 33; /* usable length of line[] */
-char      oline[33];
-char      nline[33];
+const int part_len = 33; /* usable length of line[] for most purposes */
+const int long_part_len = 129; /* length of line[] for link names and selinux contexts */
+char      oline[129];
+char      nline[129];
 const char* entry_format=        "  %-9s: %-33s, %s\n";
 const char* entry_format_justnew="  %-9s: %-33c  %s\n";
 /*************/
@@ -274,7 +275,7 @@ static int compare_str(const char *s1, const char *s2)
   - ignorelist kertoo mit ei saa vertailla
 */
 
-int compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
+DB_ATTR_TYPE compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
 {
 
 #define easy_compare(a,b) \
@@ -316,7 +317,7 @@ int compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
   easy_compare(DB_BCOUNT,bcount);
   
   if (!(DB_PERM&ignorelist)) {
-    if(l1->perm!=l2->perm){
+    if (DB_PERM&l1->attr && DB_PERM&l2->attr && l1->perm!=l2->perm) {
       ret|=DB_PERM;
     }
   } else {
@@ -379,17 +380,17 @@ void print_str_changes(char*old,char*new,const char *name)
 
   if(old==NULL){
     if(new!=NULL){
-       snprintf(oline,part_len,"<NULL>");
-       snprintf(nline,part_len,"%s",new);
+       snprintf(oline,long_part_len,"<NULL>");
+       snprintf(nline,long_part_len,"%s",new);
        ok = 1;
     }
   } else if(new==NULL){
-       snprintf(oline,part_len,"%s",old);
-       snprintf(nline,part_len,"<NULL>");
+       snprintf(oline,long_part_len,"%s",old);
+       snprintf(nline,long_part_len,"<NULL>");
        ok = 1;
    } else if(strcmp(old,new)!=0){
-        snprintf(oline,part_len,"%s",old);
-        snprintf(nline,part_len,"%s",new);
+        snprintf(oline,long_part_len,"%s",old);
+        snprintf(nline,long_part_len,"%s",new);
         ok = 1;
   }
    if(ok)
@@ -461,7 +462,7 @@ void print_single_xattrs(xattrs_type* xattrs)
     size_t num = 0;
     int width = 0;
     
-    error(2,"num=%d\n", xattrs->num);
+    error(2,"num=%lu\n", (unsigned long)xattrs->num);
 
     width = log10(xattrs->num); /* make them the same width */
     
@@ -594,11 +595,11 @@ void print_int_changes(const char* name, int old, int new, int justnew)
 }
 void print_long_changes(const char* name, AIDE_OFF_TYPE old, AIDE_OFF_TYPE new, int justnew)
 {
-#if AIDE_OFF_TYPE == off64_t
+#if SIZEOF_OFF64_T == SIZEOF_LONG_LONG
   if (!justnew) {
-    snprintf(oline,part_len,"%llu",old);
+    snprintf(oline,part_len,"%llu",(long long unsigned)old);
   }
-  snprintf(nline,part_len,"%llu",new);
+  snprintf(nline,part_len,"%llu",(long long unsigned)new);
 #else
   if (!justnew) {
     snprintf(oline,part_len,"%lu",old);
@@ -625,6 +626,234 @@ void print_string_changes(const char* name, const char* old, const char* new, in
   }
 }
 
+char get_file_type_char(mode_t mode) {
+    if (S_ISREG(mode)) return 'f';
+    else if(S_ISDIR(mode)) return 'd';
+#ifdef S_ISFIFO
+    else if (S_ISFIFO(mode)) return 'F';
+#endif
+    else if (S_ISLNK(mode)) return 'L';
+    else if (S_ISBLK(mode)) return 'B';
+    else if (S_ISCHR(mode)) return 'D';
+#ifdef S_ISSOCK
+    else if (S_ISSOCK(mode)) return 's';
+#endif
+#ifdef S_ISDOOR
+    else if (S_ISDOOR(mode)) return '|';
+#endif
+    else return '?';
+}
+
+char* get_file_type_string(mode_t mode) {
+    switch (get_file_type_char(mode)) {
+        case 'f': return "File";
+        case 'd': return "Directory";
+        case 'F': return "FIFO";
+        case 'L': return "Link";
+        case 'B': return "Block device";
+        case 'D': return "Character device";
+        case 's': return "Socket";
+        case '|': return "Door";
+        default: return "Unknown file type";
+    }
+}
+
+void print_added_line(db_line* data) {
+    if(conf->summarize_changes==1) {
+        error(2,"%c+++++++++++++++: %s\n",get_file_type_char(data->perm) , data->filename);
+    } else {
+        error(2,"added: %s\n",data->filename);
+    }
+}
+
+void print_removed_line(db_line* data) {
+    if(conf->summarize_changes==1) {
+        error(2,"%c---------------: %s\n",get_file_type_char(data->perm), data->filename);
+    } else {
+        error(2,"removed: %s\n",data->filename);
+    }
+}
+
+int str_has_changed(char*old,char*new)
+{
+    return (((old!=NULL && new!=NULL) &&
+                strcmp(old,new)!=0 ) &&
+            (old!=NULL || new!=NULL));
+}
+
+int md_has_changed(byte*old,byte*new,int len)
+{
+    return (((old!=NULL && new!=NULL) &&
+                (bytecmp(old,new,len)!=0)) || 
+            ((old!=NULL && new==NULL) || 
+             (old==NULL && new!=NULL)));
+}
+
+char get_size_char(int ignorelist, db_line* old, db_line* new) {
+    if (DB_SIZE&old->attr || DB_SIZEG&old->attr) {
+        if ((DB_SIZE&old->attr && DB_SIZE&new->attr) || (DB_SIZEG&old->attr && DB_SIZEG&new->attr)) {
+            if (((DB_SIZE&old->attr && DB_SIZE&new->attr) && DB_SIZE&ignorelist) || 
+                    ((DB_SIZEG&old->attr && DB_SIZEG&new->attr) && DB_SIZEG&ignorelist)) {
+                return ':';
+            } else if (old->size < new->size) {
+                return '>';
+            } else if (old->size > new->size) {
+                return '<';
+            } else {
+                return '=';
+            }
+        } else {
+            return '-';
+        }
+    } else if ((!(DB_SIZE&old->attr) && DB_SIZE&new->attr) || (!(DB_SIZEG&old->attr) && DB_SIZEG&new->attr) ) {
+        return '+';
+    } else {
+        return ' ';
+    }
+}
+
+void print_changed_line(db_line* old,db_line* new, DB_ATTR_TYPE ignorelist) {
+
+#define easy_compare_char(a,b,c,d) \
+    if (a&old->attr) { \
+        if (a&new->attr) { \
+            if (a&ignorelist) { \
+                summary[d]=':'; \
+            } else if (b) { \
+                summary[d]=c; \
+            } else { \
+                summary[d]='.'; \
+            } \
+        } else { \
+            summary[d]='-'; \
+        } \
+    } else if (a&new->attr) { \
+        summary[d]='+'; \
+    }
+
+#define easy_char(a,b,c,d) \
+    if (a&old->attr) { \
+        if (a&new->attr) { \
+            if (a&ignorelist) { \
+                summary[d]=':'; \
+            } else if (old->b!=new->b) { \
+                summary[d]=c; \
+            } else { \
+                summary[d]='.'; \
+            } \
+        } else { \
+            summary[d]='-'; \
+        } \
+    } else if (a&new->attr) { \
+        summary[d]='+'; \
+    }
+
+    if(conf->summarize_changes==1) {
+        char summary[]="                ";
+        summary[0]=get_file_type_char(new->perm);
+        easy_compare_char(DB_LINKNAME,str_has_changed(old->linkname,new->linkname),'l',1);
+        summary[2]=get_size_char(ignorelist, old, new);
+        easy_char(DB_BCOUNT,bcount,'b',3);
+        easy_char(DB_PERM,perm,'p',4);
+        easy_char(DB_UID,uid,'u',5);
+        easy_char(DB_GID,gid,'g',6);
+        easy_char(DB_ATIME,atime,'a',7);
+        easy_char(DB_MTIME,mtime,'m',8);
+        easy_char(DB_CTIME,ctime,'c',9);
+        easy_char(DB_INODE,inode,'i',10);
+        easy_char(DB_LNKCOUNT,nlink,'n',11);
+        if (
+                ((DB_MD5&old->attr && DB_MD5&new->attr) && md_has_changed(old->md5,new->md5,HASH_MD5_LEN) && !(DB_MD5&ignorelist)) ||
+                ((DB_SHA1&old->attr &&  DB_SHA1&new->attr) && md_has_changed(old->sha1,new->sha1,HASH_SHA1_LEN) && !(DB_SHA1&ignorelist)) ||
+                ((DB_RMD160&old->attr && DB_RMD160&new->attr) && md_has_changed(old->rmd160,new->rmd160,HASH_RMD160_LEN) && !(DB_RMD160&ignorelist)) ||
+                ((DB_TIGER&old->attr && DB_TIGER&new->attr) && md_has_changed(old->tiger,new->tiger,HASH_TIGER_LEN) && !(DB_TIGER&ignorelist)) ||
+                ((DB_SHA256&old->attr && DB_SHA256&new->attr) && md_has_changed(old->sha256,new->sha256,HASH_SHA256_LEN) && !(DB_SHA256&ignorelist)) ||
+                ((DB_SHA512&old->attr && DB_SHA512&new->attr) && md_has_changed(old->sha512,new->sha512,HASH_SHA512_LEN) && !(DB_SHA512&ignorelist)) 
+#ifdef WITH_MHASH
+                || ((DB_CRC32&old->attr && DB_CRC32&new->attr) && md_has_changed(old->crc32,new->crc32,HASH_CRC32_LEN) && !(DB_CRC32&ignorelist)) ||
+                ((DB_HAVAL&old->attr && DB_HAVAL&new->attr) && md_has_changed(old->haval,new->haval,HASH_HAVAL256_LEN) && !(DB_HAVAL&ignorelist)) ||
+                ((DB_GOST&old->attr && DB_GOST&new->attr) && md_has_changed(old->gost,new->gost,HASH_GOST_LEN) && !(DB_GOST&ignorelist)) ||
+                ((DB_CRC32B&old->attr && DB_CRC32B&new->attr) && md_has_changed(old->crc32b,new->crc32b,HASH_CRC32B_LEN) && !(DB_CRC32B&ignorelist)) ||
+                ((DB_WHIRLPOOL&old->attr && DB_WHIRLPOOL&new->attr) && md_has_changed(old->whirlpool,new->whirlpool,HASH_WHIRLPOOL_LEN && !(DB_WHIRLPOOL&ignorelist)))
+#endif        
+           ) {
+            summary[12]='C';
+        } else if (
+                ((DB_MD5&old->attr) && !(DB_MD5&new->attr)) ||
+                ((DB_SHA1&old->attr) &&  !(DB_SHA1&new->attr)) ||
+                ((DB_RMD160&old->attr) && !(DB_RMD160&new->attr)) ||
+                ((DB_TIGER&old->attr) && !(DB_TIGER&new->attr)) ||
+                ((DB_SHA256&old->attr) && !(DB_SHA256&new->attr)) ||
+                ((DB_SHA512&old->attr) && !(DB_SHA512&new->attr))
+#ifdef WITH_MHASH
+                || ((DB_CRC32&old->attr) && !(DB_CRC32&new->attr)) ||
+                ((DB_HAVAL&old->attr) && !(DB_HAVAL&new->attr)) ||
+                ((DB_GOST&old->attr) && !(DB_GOST&new->attr)) ||
+                ((DB_CRC32B&old->attr) && !(DB_CRC32B&new->attr)) ||
+                ((DB_WHIRLPOOL&old->attr) && !(DB_WHIRLPOOL&new->attr))
+#endif        
+                ) {
+            summary[12]='-';
+        } else if (
+                (!(DB_MD5&old->attr) && (DB_MD5&new->attr)) ||
+                (!(DB_SHA1&old->attr) &&  (DB_SHA1&new->attr)) ||
+                (!(DB_RMD160&old->attr) && (DB_RMD160&new->attr)) ||
+                (!(DB_TIGER&old->attr) && (DB_TIGER&new->attr)) ||
+                (!(DB_SHA256&old->attr) && (DB_SHA256&new->attr)) ||
+                (!(DB_SHA512&old->attr) && (DB_SHA512&new->attr))
+#ifdef WITH_MHASH
+                || (!(DB_CRC32&old->attr) && (DB_CRC32&new->attr)) ||
+                (!(DB_HAVAL&old->attr) && (DB_HAVAL&new->attr)) ||
+                (!(DB_GOST&old->attr) && (DB_GOST&new->attr)) ||
+                (!(DB_CRC32B&old->attr) && (DB_CRC32B&new->attr)) ||
+                (!(DB_WHIRLPOOL&old->attr) && (DB_WHIRLPOOL&new->attr))
+#endif        
+                ) {
+            summary[12]='+';
+        } else if (
+                ((DB_MD5&old->attr && DB_MD5&new->attr) && DB_MD5&ignorelist) ||
+                ((DB_SHA1&old->attr &&  DB_SHA1&new->attr) && DB_SHA1&ignorelist) ||
+                ((DB_RMD160&old->attr && DB_RMD160&new->attr) && DB_RMD160&ignorelist) ||
+                ((DB_TIGER&old->attr && DB_TIGER&new->attr) && DB_TIGER&ignorelist) ||
+                ((DB_SHA256&old->attr && DB_SHA256&new->attr) && DB_SHA256&ignorelist) ||
+                ((DB_SHA512&old->attr && DB_SHA512&new->attr) && DB_SHA512&ignorelist)
+#ifdef WITH_MHASH
+                || ((DB_CRC32&old->attr && DB_CRC32&new->attr) && DB_CRC32&ignorelist) ||
+                ((DB_HAVAL&old->attr && DB_HAVAL&new->attr) && DB_HAVAL&ignorelist) ||
+                ((DB_GOST&old->attr && DB_GOST&new->attr) && DB_GOST&ignorelist) ||
+                ((DB_CRC32B&old->attr && DB_CRC32B&new->attr) && DB_CRC32B&ignorelist) ||
+                ((DB_WHIRLPOOL&old->attr && DB_WHIRLPOOL&new->attr) && DB_WHIRLPOOL&ignorelist)
+#endif        
+                ) {
+            summary[12]=':';
+        } else if (
+                (DB_MD5&old->attr && DB_MD5&new->attr) ||
+                (DB_SHA1&old->attr &&  DB_SHA1&new->attr) ||
+                (DB_RMD160&old->attr && DB_RMD160&new->attr) ||
+                (DB_TIGER&old->attr && DB_TIGER&new->attr) ||
+                (DB_SHA256&old->attr && DB_SHA256&new->attr) ||
+                (DB_SHA512&old->attr && DB_SHA512&new->attr)
+#ifdef WITH_MHASH
+                || (DB_CRC32&old->attr && DB_CRC32&new->attr) ||
+                (DB_HAVAL&old->attr && DB_HAVAL&new->attr) ||
+                (DB_GOST&old->attr && DB_GOST&new->attr) ||
+                (DB_CRC32B&old->attr && DB_CRC32B&new->attr) ||
+                (DB_WHIRLPOOL&old->attr && DB_WHIRLPOOL&new->attr)
+#endif        
+                ) {
+            summary[12]='.';
+        }
+#ifdef WITH_ACL
+        easy_compare_char(DB_ACL,compare_acl(old->acl,new->acl)==RETFAIL,'A',13);
+#endif
+        easy_compare_char(DB_XATTRS,compare_xattrs(old->xattrs,new->xattrs)==RETFAIL,'X',14);
+        easy_compare_char(DB_SELINUX,str_has_changed(old->cntx,new->cntx),'S',15);
+
+        error(2,"%s: %s\n",summary, new->filename);
+    } else {
+        error(2,"changed: %s\n",new->filename);
+    }
+}
 
 void print_dbline_changes(db_line* old,db_line* new,
                           DB_ATTR_TYPE ignorelist,DB_ATTR_TYPE forced_attrs)
@@ -637,11 +866,7 @@ void print_dbline_changes(db_line* old,db_line* new,
   */
   forced_attrs&=new->attr;
   
-  if(S_ISDIR(new->perm_o)){
-    error(2,"\nDirectory: %s\n",old->filename);
-  }else {
-    error(2,"\nFile: %s\n",old->filename);
-  }
+  error(2,"\n%s: %s\n",get_file_type_string(new->perm),new->filename);
   
   if(!(DB_LINKNAME&ignorelist)){
     print_str_changes(old->linkname,new->linkname, "Lname");
@@ -810,7 +1035,7 @@ void init_rxlst(list* rxlst)
     
     if (data==NULL){
       error(0,_("Not enough memory for regexpr compile... exiting..\n"));
-      abort();
+      exit(EXIT_FAILURE);
     }
     
     strcpy(data+1,((rx_rule*)r->data)->rx);
@@ -821,7 +1046,7 @@ void init_rxlst(list* rxlst)
     rxrultmp->conf_lineno=-1;
     rxtmp=(regex_t*)malloc(sizeof(regex_t));
     if( regcomp(rxtmp,data,REG_EXTENDED|REG_NOSUB)){
-      error(0,_("Error in selective regexp:%s"),((rx_rule*)r->data)->rx);
+      error(0,_("Error in selective regexp: %s\n"),((rx_rule*)r->data)->rx);
       free(data);
     }else {
       rxrultmp->conf_lineno=((rx_rule*)r)->conf_lineno;
@@ -879,7 +1104,7 @@ void print_report_footer(struct tm* st)
 	  st->tm_hour, st->tm_min, st->tm_sec);
 }
 
-void compare_db(list* new,db_config* conf)
+void compare_db(list* new,db_config* dbconf)
 {
   db_line* old=NULL;
   list* l=new;
@@ -903,14 +1128,14 @@ void compare_db(list* new,db_config* conf)
 
 
   /* With this we avoid unnecessary checking of removed files. */
-  if(conf->action&DO_INIT){
+  if(dbconf->action&DO_INIT){
     initdbwarningprinted=1;
   } else {
     /* We have to init the rxlsts since they are copied and then 
        initialized in gen_list.c */
-    init_rxlst(conf->selrxlst);
-    init_rxlst(conf->equrxlst);
-    init_rxlst(conf->negrxlst);
+    init_rxlst(dbconf->selrxlst);
+    init_rxlst(dbconf->equrxlst);
+    init_rxlst(dbconf->negrxlst);
   }
   
   /* We have a way to ignore some changes... */ 
@@ -931,11 +1156,11 @@ void compare_db(list* new,db_config* conf)
       /* FIXME There should be a check for this in changed part also */
       /* This part should also be rethinked */
       if(!initdbwarningprinted &&
-	 (check_list_for_match(conf->selrxlst,old->filename,&tempignore) ||
-	  check_list_for_match(conf->equrxlst,old->filename,&tempignore)) &&
-	 !check_list_for_match(conf->negrxlst,old->filename,&tempignore)){
-	if(!(conf->action&DO_INIT)){
-	  error(2,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
+	 (check_list_for_match(dbconf->selrxlst,old->filename,&tempignore) ||
+	  check_list_for_match(dbconf->equrxlst,old->filename,&tempignore)) &&
+	 !check_list_for_match(dbconf->negrxlst,old->filename,&tempignore)){
+	if(!(dbconf->action&DO_INIT)){
+	  error(2,_("WARNING: Old db contains a entry that shouldn\'t be there, run --init or --update\n"));
 	}
 	initdbwarningprinted=1;
       }
@@ -945,7 +1170,7 @@ void compare_db(list* new,db_config* conf)
       int localignorelist=old->attr ^ ((db_line*)r->data)->attr;
       
       if ((localignorelist&(~(DB_NEWFILE|DB_RMFILE)))!=0) {
-	error(2,"File %s in databases has different attributes, %llx,%llx\n",
+	error(2,"Entry %s in databases has different attributes: %llx %llx\n",
               old->filename,old->attr,((db_line*)r->data)->attr);
       }
       
@@ -990,8 +1215,8 @@ void compare_db(list* new,db_config* conf)
       error(2,_("Added files:\n"));
       error(2,_("---------------------------------------------------\n\n"));
       for(r=added;r;r=r->next){
-	error(2,"added: %s\n",((db_line*)r->data)->filename);
-	if(conf->verbose_level<20){
+       print_added_line((db_line*)r->data);
+	if(dbconf->verbose_level<20){
 	  if(S_ISDIR(((db_line*)r->data)->perm)){
 	    eat_files_indir(r->next,((db_line*)r->data)->filename,&filesindir);
 	    if(filesindir>0){
@@ -1011,7 +1236,7 @@ void compare_db(list* new,db_config* conf)
       error(2,_("Removed files:\n"));
       error(2,_("---------------------------------------------------\n\n"));
       for(r=removed;r;r=r->next){
-	error(2,"removed: %s\n",((db_line*)r->data)->filename);
+       print_removed_line((db_line*)r->data);
       }
     }
 
@@ -1019,12 +1244,13 @@ void compare_db(list* new,db_config* conf)
       error(2,_("\n---------------------------------------------------\n"));
       error(2,_("Changed files:\n"));
       error(2,_("---------------------------------------------------\n\n"));
-      for(r=changedold;r;r=r->next){
-	error(2,"changed: %s\n",((db_line*)r->data)->filename);
+      for(r=changedold,l=changednew;r;r=r->next,l=l->next){
+	print_changed_line((db_line*)r->data,
+			     (db_line*)l->data,ignorelist);
       }
     }
 
-    if((conf->verbose_level>=5)&&(nchg!=0)){
+    if((dbconf->verbose_level>=5)&&(nchg!=0)){
       error(2,_("\n---------------------------------------------------\n"));
       error(2,_("Detailed information about changes:\n"));
       error(2,_("---------------------------------------------------\n\n"));
@@ -1035,8 +1261,8 @@ void compare_db(list* new,db_config* conf)
 			     (db_line*)l->data,localignorelist,forced_attrs);
       }
     }
-    conf->end_time=time(&(conf->end_time));
-    print_report_footer(localtime(&(conf->end_time)));
+    dbconf->end_time=time(&(dbconf->end_time));
+    print_report_footer(localtime(&(dbconf->end_time)));
   }
 }
 
@@ -1065,7 +1291,7 @@ void send_audit_report(long nadd, long nrem, long nchg)
 #endif /* WITH_AUDIT */
 }
 
-long report_tree(seltree* node,int stage, long* stat)
+long report_tree(seltree* node,int stage, long* status)
 {
   list* r=NULL;
   int ignorelist=0;
@@ -1075,8 +1301,8 @@ long report_tree(seltree* node,int stage, long* stat)
   ignorelist=get_ignorelist();
   forced_attrs=get_report_attributes();
   
-  if(stat[0]){
-    stat[0]=0;
+  if(status[0]){
+    status[0]=0;
     top=1;
   }
 
@@ -1090,7 +1316,7 @@ long report_tree(seltree* node,int stage, long* stat)
        If checked == 0 there is nothing to report
     */
     if(node->checked!=0){
-      stat[1]++;
+      status[1]++;
       if((node->checked&DB_OLD)&&(node->checked&DB_NEW)&&
 	 (node->old_data==NULL)&&(node->new_data==NULL)){
 	/* Node was added to twice and discovered to be not changed*/
@@ -1098,69 +1324,69 @@ long report_tree(seltree* node,int stage, long* stat)
 	/* File is in new db but not old. (ADDED) */
 	/* unless it was moved in */
 	if (!((node->checked&NODE_ALLOW_NEW)||(node->checked&NODE_MOVED_IN))) {
-	  stat[2]++;
+	  status[2]++;
 	  node->checked|=NODE_ADDED;
 	}
       }else if((node->checked&DB_OLD)&&!(node->checked&DB_NEW)){
 	/* File is in old db but not new. (REMOVED) */
 	/* unless it was moved out */
 	if (!((node->checked&NODE_ALLOW_RM)||(node->checked&NODE_MOVED_OUT))) {
-	  stat[3]++;
+	  status[3]++;
 	  node->checked|=NODE_REMOVED;
 	}
       }else if((node->checked&DB_OLD)&&(node->checked&DB_NEW)&&
 	       (node->old_data!=NULL)&&(node->new_data!=NULL)){
 	/* File is in both db's and the data is still there. (CHANGED) */
 	if(!(node->checked&(NODE_MOVED_IN|NODE_MOVED_OUT))){
-	  stat[4]++;
+	  status[4]++;
 	  node->checked|=NODE_CHANGED;
 	}else if (!((node->checked&NODE_ALLOW_NEW)||(node->checked&NODE_MOVED_IN))) {
-	  stat[2]++;
+	  status[2]++;
 	  node->checked|=NODE_ADDED;
 	}else if (!((node->checked&NODE_ALLOW_RM)||(node->checked&NODE_MOVED_OUT))) {
-	  stat[3]++;
+	  status[3]++;
 	  node->checked|=NODE_REMOVED;
 	}
       }
     }
   }
 
-  if((stage==1)&&stat[2]){
+  if((stage==1)&&status[2]){
     if(top){
       error(2,_("\n---------------------------------------------------\n"));
       error(2,_("Added files:\n"));
       error(2,_("---------------------------------------------------\n\n"));
     }
     if(node->checked&NODE_ADDED){
-      error(2,_("added: %s\n"),node->new_data->filename);
+      print_added_line(node->new_data);
     }
   }
 
-  if((stage==2)&&stat[3]){
+  if((stage==2)&&status[3]){
     if(top){
       error(2,_("\n---------------------------------------------------\n"));
       error(2,_("Removed files:\n"));
       error(2,_("---------------------------------------------------\n\n"));
     }
     if(node->checked&NODE_REMOVED){
-      error(2,_("removed: %s\n"),node->old_data->filename);
+      print_removed_line(node->old_data);
     }
   }
 
-  if((stage==3)&&stat[4]){
+  if((stage==3)&&status[4]){
     if(top){
       error(2,_("\n---------------------------------------------------\n"));
       error(2,_("Changed files:\n"));
       error(2,_("---------------------------------------------------\n\n"));
     }
     if(node->checked&NODE_CHANGED){
-      error(2,_("changed: %s\n"),node->new_data->filename);
+      print_changed_line(node->old_data,node->new_data,ignorelist);
     }
   }
 
-  if((stage==4)&&(conf->verbose_level>=5)&&stat[4]){
+  if((stage==4)&&(conf->verbose_level>=5)&&status[4]){
     if(top){
-      error(2,_("\n--------------------------------------------------\n"));
+      error(2,_("\n---------------------------------------------------\n"));
       error(2,_("Detailed information about changes:\n"));
       error(2,_("---------------------------------------------------\n\n"));
     }
@@ -1172,15 +1398,15 @@ long report_tree(seltree* node,int stage, long* stat)
 
   /* All stage dependent things done for this node. Let's check children */
   for(r=node->childs;r;r=r->next){
-    report_tree((seltree*)r->data,stage,stat);
+    report_tree((seltree*)r->data,stage,status);
   }
 
-  if(top&&(stage==0)&&((stat[2]+stat[3]+stat[4])>0)){
-    send_audit_report(stat[2],stat[3],stat[4]);
-    print_report_header(stat[1],stat[2],stat[3],stat[4]);
+  if(top&&(stage==0)&&((status[2]+status[3]+status[4])>0)){
+    send_audit_report(status[2],status[3],status[4]);
+    print_report_header(status[1],status[2],status[3],status[4]);
   }
   
-  return (stat[2]+stat[3]+stat[4]);
+  return (status[2]+status[3]+status[4]);
 }
 
 const char* aide_key_9=CONFHMACKEY_09;
