@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/gen_list.c,v 1.30 2006/12/08 22:00:37 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -61,6 +61,7 @@ void no_hash(db_line* line);
 char* strrxtok(char* rx)
 {
   char*p=NULL;
+  char*t=NULL;
   size_t i=0;
 
   /* The following code assumes that the first character is a slash */
@@ -83,8 +84,10 @@ char* strrxtok(char* rx)
 	i=strlen(p);
 	break;
       case '\\':
-	strcpy(p+i,p+i+1);
-	i++;
+	t=strdup(p);
+	strcpy(p+i,t+i+1);
+	free(t);
+	t=NULL;
 	break;
       default:
 	break;
@@ -321,7 +324,7 @@ void gen_seltree(list* rxlist,seltree* tree,char type)
     
     rxtmp=(regex_t*)malloc(sizeof(regex_t));
     if( regcomp(rxtmp,data,REG_EXTENDED|REG_NOSUB)){
-      error(0,_("Error in selective regexp:%s"),curr_rule->rx);
+      error(0,_("Error in selective regexp: %s\n"),curr_rule->rx);
       free(data);
     }else{
       /* replace regexp text with regexp compiled */
@@ -415,8 +418,8 @@ static void xattrs2line(db_line *line)
     xatrs = realloc(xatrs, xsz);
   }
 
-  if ((xret == -1) && (errno == ENOSYS))
-  { /* do nothing */ }
+  if ((xret == -1) && ((errno == ENOSYS) || (errno == ENOTSUP)))
+  {   line->attr&=(~DB_XATTRS); }
   else if (xret == -1)
     error(0, "listxattrs failed for %s:%m\n", line->filename);
   else if (xret)
@@ -471,8 +474,10 @@ static void selinux2line(db_line *line)
 
   if (lgetfilecon_raw(line->filename, &cntx) == -1)
   {
-    error(0, "lgetfilecon_raw failed for %s:%m\n", line->filename);
-    return;
+      line->attr&=(~DB_SELINUX);
+      if ((errno != ENOATTR) && (errno != EOPNOTSUPP))
+          error(0, "lgetfilecon_raw failed for %s:%m\n", line->filename);
+      return;
   }
 
   line->cntx = strdup(cntx);
@@ -550,9 +555,8 @@ list* add_file_to_list(list* listp,char*filename,DB_ATTR_TYPE attr,int* addok)
 #endif
     
     if(conf->warn_dead_symlinks==1) {
-      struct AIDE_STAT_TYPE fs;
-      int sres;
-      sres=AIDE_STAT_FUNC(fil->filename,&fs);
+      struct AIDE_STAT_TYPE tmp_fs;
+      sres=AIDE_STAT_FUNC(fil->filename,&tmp_fs);
       if (sres!=0) {
 	error(4,"Dead symlink detected at %s\n",fil->filename);
       }
@@ -627,11 +631,7 @@ list* add_file_to_list(list* listp,char*filename,DB_ATTR_TYPE attr,int* addok)
     fil->gid=0;
   }
 
-  if(DB_PERM&fil->attr){
-    fil->perm=fs.st_mode;
-  }else{
-    fil->perm=0;
-  }
+  fil->perm=fs.st_mode;
 
   if(DB_SIZE&fil->attr||DB_SIZEG&fil->attr){
     fil->size=fs.st_size;
@@ -732,33 +732,6 @@ int check_list_for_match(list* rxrlist,char* text,DB_ATTR_TYPE* attr)
   return retval;
 }
 
-//this is used to check if $text if equal to a node in $rxrlist
-//should be used to check equ_rx_lst only
-int check_list_for_equal(list* rxrlist,char* text,DB_ATTR_TYPE* attr)
-{
-  list* r=NULL;
-  int retval=1;
-  char *temp;
-
-  for(r=rxrlist;r;r=r->next){
-    temp=((rx_rule*)r->data)->rx;
-    
-    //FIXME, if rx not begin with ^, may need to do something else
-    if(temp[0]=='^') //^ is for reg exp, we can ignore this character
-      temp++;
-
-    //we don't need to worry about buff-overflow, so strcmp is safe
-    if((retval=strcmp(temp, text))==0){
-      *attr=((rx_rule*)r->data)->attr;
-      error(231,"\"%s\" matches string from line #%ld: %s\n",text,((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
-      break;
-    } else {
-      error(231,"\"%s\" doesn't match string from line #%ld: %s\n",text,((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
-    }
-  }
-  return retval;
-}
-
 /* 
  * Function check_node_for_match()
  * calls itself recursively to go to the top and then back down.
@@ -783,35 +756,24 @@ int check_node_for_match(seltree*node,char*text,int retval,DB_ATTR_TYPE* attr)
     return retval;
   }
   
-  /* We need this to check whether this was the first one *
-   * to be called and not a recursive call */
-  if(!((retval&16)==16)){
-    retval|=16;
+  /* if this call is not recursive we check the equals list and we set top *
+   * and retval so we know following calls are recursive */
+  if(!(retval&16)){
     top=1;
-  } else {
-    top=0;
-  }
-  
-  /* if no deeper match found */
-  if(!((retval&8)==8)&&!((retval&4)==4)){
+    retval|=16;
+
     if(!check_list_for_match(node->equ_rx_lst,text,attr)){
-      /*
-	Zhi Wen Wong added this line to fix bug that equ not work for 
-	compare
-	if we do "=/bin", we should only check /bin
-	so, /bin/bash or /bin/something should return 0 as neg
-      */
-      if(!check_list_for_equal(node->equ_rx_lst,text,attr))
-	retval|=(2|4);
-    };
-  };
+      retval|=2|4;
+    }
+  }
   /* We'll use retval to pass information on whether to recurse 
    * the dir or not */
 
 
-  if(!((retval&8)==8)&&!((retval&4)==4)){
+  /* If 4 and 8 are not set, we will check for matches */
+  if(!(retval&(4|8))){
     if(!check_list_for_match(node->sel_rx_lst,text,attr))
-      retval|=(1|8);
+      retval|=1|8;
   }
 
   /* Now let's check the ancestors */
@@ -987,16 +949,16 @@ void print_tree(seltree* tree) {
 
   for(r=tree->sel_rx_lst;r!=NULL;r=r->next) {
 	rxc=r->data;
-	error(246,"%i\t%s\n",rxc->conf_lineno,rxc->rx);
+	error(246,"%li\t%s\n",rxc->conf_lineno,rxc->rx);
   }
   for(r=tree->equ_rx_lst;r!=NULL;r=r->next) {
         rxc=r->data;
-        error(246,"%i=\t%s\n",rxc->conf_lineno,rxc->rx);
+        error(246,"%li=\t%s\n",rxc->conf_lineno,rxc->rx);
   }
   
   for(r=tree->neg_rx_lst;r!=NULL;r=r->next) {
 	  rxc=r->data;
-	  error(246,"%i!\t%s\n",rxc->conf_lineno,rxc->rx);
+	  error(246,"%li!\t%s\n",rxc->conf_lineno,rxc->rx);
   }
   
   for(r=tree->childs;r!=NULL;r=r->next) {
@@ -1048,9 +1010,7 @@ void strip_dbline(db_line* line,DB_ATTR_TYPE attr)
   if(!(attr&DB_LINKNAME)){
     checked_free(line->linkname);
   }
-  if(!(attr&DB_PERM)){
-    line->perm=0;
-  }
+  /* permissions are always needed for summarize_changes, hence it is never stripped */
   if(!(attr&DB_UID)){
     line->uid=0;
   }
@@ -1194,7 +1154,7 @@ void add_file_to_tree(seltree* tree,db_line* file,int db,int status,
   if((node->checked&DB_OLD)&&(node->checked&DB_NEW)){
     localignorelist=(node->new_data->attr^node->old_data->attr);
     if (localignorelist!=0) {
-      error(2,"File %s in databases has different attributes, %llx,%llx\n",
+      error(2,"Entry %s in databases has different attributes: %llx %llx\n",
             node->old_data->filename,node->old_data->attr,node->new_data->attr);
     }
     
@@ -1243,7 +1203,7 @@ void add_file_to_tree(seltree* tree,db_line* file,int db,int status,
       localignorelist=(oldData->attr^newData->attr)&(~(DB_NEWFILE|DB_RMFILE));
 
       if (localignorelist!=0) {
-        error(5,"File \"%s\" \"%s\" in databases has different attributes (here3), %llx,%llx\n",
+        error(5,"Entry \"%s\" \"%s\" in databases has different attributes (here3): %llx %llx\n",
   	    newData->filename,oldData->filename,oldData->attr,newData->attr);
       }
     
@@ -1354,10 +1314,10 @@ db_line* get_file_attrs(char* filename,DB_ATTR_TYPE attr)
   memset(line,0,sizeof(db_line));
   
   /*
-    We want filename and linkname
+    We want filename
   */
 
-  line->attr=attr|DB_FILENAME|DB_LINKNAME;
+  line->attr=attr|DB_FILENAME;
   
   /*
     Just copy some needed fields.
@@ -1448,7 +1408,7 @@ void populate_tree(seltree* tree)
           free(old);
           old=NULL;
           if(!initdbwarningprinted){
-	    error(3,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
+	    error(3,_("WARNING: Old db contains a entry that shouldn\'t be there, run --init or --update\n"));
 	    initdbwarningprinted=1;
 	  }
 	}
@@ -1562,10 +1522,8 @@ void hsymlnk(db_line* line) {
      * We use realloc :)
      */
     line->linkname=realloc(line->linkname,len+1);
-    line->attr|=DB_LINKNAME;
-  }else {
-    /* Just remove linkname avaivilibity bit from this entry */
-    line->attr&=(~DB_LINKNAME); 
+  } else {
+      line->attr&=(~DB_LINKNAME);
   }
   
 }
