@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/list.c,v 1.3 2006/10/27 20:44:38 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,7 +47,7 @@
 
 
 /* 
- * Some way to handle mallocs failure would be nice. Now it say abort().
+ * Some way to handle mallocs failure would be nice.
  */
 
 list* list_append(list* listp,void*data)
@@ -57,7 +57,7 @@ list* list_append(list* listp,void*data)
 
   if (newitem==NULL) {
     error(0,"Not enough memory to add a new item to list.\n");
-    abort();
+    exit(EXIT_FAILURE);
   }
   
   if(listp==NULL){
@@ -65,7 +65,7 @@ list* list_append(list* listp,void*data)
     
     if (header==NULL){
       error(0,"Not enough memory for list header allocation\n");
-      abort();
+      exit(EXIT_FAILURE);
     }
     
     newitem->data=data;
