@@ -1,7 +1,7 @@
 /* Aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/md.c,v 1.6 2006/11/29 12:58:02 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -201,6 +201,12 @@ int init_md(struct md_container* md) {
 #endif 
 #ifdef WITH_GCRYPT
   error(255,"Gcrypt library initialization\n");
+  	if(!gcry_check_version(GCRYPT_VERSION)) {
+		error(0,"libgcrypt version mismatch\n");
+		exit(VERSION_MISMATCH_ERROR);
+	}
+	gcry_control(GCRYCTL_DISABLE_SECMEM, 0);
+	gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
 	if(gcry_md_open(&md->mdh,0,0)!=GPG_ERR_NO_ERROR){
 		error(0,"gcrypt_md_open failed\n");
 		exit(IO_ERROR);
@@ -340,9 +346,9 @@ void md2line(struct md_container* md,struct db_line* line) {
 
 #define copyhash(a,b,c)        \
   if (line->attr&a) {          \
-    error(255,"Line has %i\n",a); \
+    error(255,"Line has %llu\n",a); \
     if (md->calc_attr&a) {     \
-      error(255,"copying %i\n",a); \
+      error(255,"copying %llu\n",a); \
       line->b=(byte*)malloc(c);       \
       memcpy(line->b,md->b,c); \
     } else {                   \
