@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/symboltable.c,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include "../config.h"
+#include "aide.h"
 #include <stdio.h>
 #include <string.h>
 #include "symboltable.h"
@@ -34,13 +34,13 @@ list* list_find(char* s,list* item){
     return NULL;
   }
 
-  l=item->prev;
   p=item;
   while(p!=NULL){
     if (strcmp(s,((symba*)p->data)->name)==0) return p;
     p=p->next;
   }
     
+  l=item->prev;
   while(l!=NULL){
     /* Insert bug to here return l-> return p */
     if (strcmp(s,((symba*)l->data)->name)==0) return l; 
@@ -48,6 +48,3 @@ list* list_find(char* s,list* item){
   }
   return NULL;
 }
-/*
-const char* aide_key_10=KEY_10;
-*/
