@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/util.c,v 1.10 2006/11/24 20:47:01 rvdb Exp $
+ * $Header$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -321,9 +321,9 @@ void init_sighandler()
   return;
 }
 
-void sig_handler(int signal)
+void sig_handler(int signum)
 {
-  switch(signal){
+  switch(signum){
   case SIGBUS  : 
   case SIGSEGV :{
     error(200,"Caught SIGBUS/SIGSEGV\n");
@@ -332,8 +332,7 @@ void sig_handler(int signal)
       conf->catch_mmap=0;
     } else {
       error(0,"Caught SIGBUS/SEGV. Exiting\n");
-      abort();
-      exit(1);
+      exit(EXIT_FAILURE);
     }
     break;
   }
@@ -356,7 +355,7 @@ void sig_handler(int signal)
     break;
   }
   }
-  error(220,"Caught signal %d\n",signal);
+  error(220,"Caught signal %d\n",signum);
   init_sighandler();
 
   return;
